/*
 ArdBot line following demo using
  Pololu QTR-8RC sensor array
 Requires Arduino IDE version 0017 
  or later (0019 or later preferred)
 Requires PololuQTRSensors library
  (available from Pololu; see text)
*/

#include <PololuQTRSensors.h>
#include <Servo.h> 

Servo servoLeft;          // Define left servo
Servo servoRight;         // Define right servo

#define NUM_SENSORS   5     // Number of sensors
#define TIMEOUT       2500  // Maximum timeout per sensor

// QTR-8 connected to digital pins 4, 5, 6, 11, and 12
PololuQTRSensorsRC qtrrc((unsigned char[]) {4, 5, 6, 11, 12},
  NUM_SENSORS, TIMEOUT, QTR_NO_EMITTER_PIN);
unsigned int sensorValues[NUM_SENSORS];
int variance = 300;        // Variance from center (2000)

void setup() {
  servoLeft.attach(10);  // Left servo pin D10
  servoRight.attach(9);  // Right server pin D9
  
  delay(500);
  pinMode(13, OUTPUT);
  digitalWrite(13, HIGH);      // Show calibrate mode
  for (int i = 0; i < 400; i++) {
    qtrrc.calibrate();        // Read sensors to calibrate
  }
  digitalWrite(13, LOW);      // Calibrate completed
  delay(1000);
}

void loop() {
  unsigned int position = qtrrc.readLine(sensorValues);

  if (position > (2000 - variance) && position < (2000 + variance)) {
    line_forward();
  }
  if (position > 2000 + variance) {
    line_spinLeft();    
    delay(2);
  }
  if (position < 2000 - variance) {
    line_spinRight();    
    delay(2);
  }
}

// Motion routines for line following
void line_forward() {
  servoLeft.write(0);
  servoRight.write(180);
}

void line_slipRight() {
  servoLeft.write(90);
  servoRight.write(180);
}
void line_slipLeft() {
  servoLeft.write(0);
  servoRight.write(90);
}
void line_spinRight() {
  servoLeft.write(180);
  servoRight.write(180);
}
void line_spinLeft() {
  servoLeft.write(0);
  servoRight.write(0);
}
